%{ open Ast %}

%token SEMI 
%token LPAREN 
%token RPAREN 
%token LBRACE 
%token RBRACE 
%token COMMA 
%token QUOT 
%token ASSIGN 
%token INTASSIGN 
%token LINKASSIGN 
%token FORMASSIGN 
%token INPUTASSIGN
%token IMAGEASSIGN
%token DECLARE
%token BUTTON 
%token RETURN 
%token <string> INT_LITERAL
%token <string> STRING_LITERAL
%token <string> ID
%token EOF

%left ASSIGN
%left INTASSIGN
%left LINKASSIGN
%left TITLEASSIGN
%left FORMASSIGN
%left INPUTASSIGN
%left IMAGEASSIGN

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [], [] }
 | program vdecl { ($2 :: fst $1), snd $1 }
 | program fdecl { fst $1, ($2 :: snd $1) }

fdecl:
  ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
   { { 
	 fname = $1;
	 formals = $3;
	 locals = List.rev $6;
	 body = List.rev $7 } }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
    ID                   { [$1] }
  | formal_list COMMA ID { $3 :: $1 }

vdecl_list:
    /* nothing */    { [] }
  | vdecl_list vdecl { $2 :: $1 }

vdecl:
    DECLARE ID SEMI { $2 }

stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) } 	

expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

expr:
    INT_LITERAL      { Int_Literal($1) }
	| STRING_LITERAL   { String_Literal($1) }
  | ID               { Id($1) }
  | ID ASSIGN expr   { Assign($1, $3) }
	| ID INTASSIGN expr { IntAssign($1, $3) }
	| ID LINKASSIGN expr COMMA expr { LinkAssign($1, $3, $5) }
	| ID FORMASSIGN expr COMMA expr { FormAssign($1, $3, $5) }
	| ID INPUTASSIGN expr COMMA expr COMMA expr { InputAssign($1, $3, $5, $7) } /*ID, label, maxLen/size, name*/
	| ID IMAGEASSIGN expr COMMA expr COMMA expr { ImageAssign($1, $3, $5, $7) } /*ID, Img Source, Width, Height*/
  | ID LPAREN actuals_opt RPAREN { Invoke($1, $3) }
	| BUTTON expr COMMA expr { Button ($2, $4) }
  | LPAREN expr RPAREN { $2 }

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }
